@@ -43,7 +43,7 @@ use nix::sys::stat::Mode;
 
 use super::pf_tokenizer::PfListTokenizer;
 use crate::portable::net::{htonl, ntohl};
-use crate::{runtime_error, cfor, map_runtime_error, runtime_exception::*};
+use crate::{error_coded, cfor, map_error_coded, runtime_exception::*};
 
 
 const PF_TABLE_NAME_SIZE: usize = 32;
@@ -65,6 +65,9 @@ pub enum PfCmd
 
     /// Checks if host/hostname presents in table
     Test{ hosts: Vec<String> },
+
+    /// Flushes the 
+    Flush,
 }
 
 
@@ -189,21 +192,21 @@ impl pf_addr
                 let p =
                     match addr.char_indices().nth(pos+1)  // skip '/'
                     {
-                        None => runtime_error!("can not get char indeces at: '{}' for: '{}'", pos, addr),
+                        None => error_coded!(PfErrCode::InternalError, "can not get char indeces at: '{}' for: '{}'", pos, addr),
                         Some((idx, _)) => 
                         {
                             let prefix = 
                                 i32::from_str_radix(&addr[idx..], 10).map_err(
-                                    |e| map_runtime_error!("can not decode: '{}' to i32, {}", &addr[idx..], e)
+                                    |e| map_error_coded!(PfErrCode::InternalError, "can not decode: '{}' to i32, {}", &addr[idx..], e)
                                 )?;
 
                             if prefix < 0
                             {
-                                runtime_error!("decoded val: '{}' < minval: 0", prefix);
+                                error_coded!(PfErrCode::Einval, "decoded val: '{}' < minval: 0", prefix);
                             }
                             else if prefix > 128
                             {
-                                runtime_error!("decoded val: '{}' > maxval: 128", prefix);
+                                error_coded!(PfErrCode::Einval, "decoded val: '{}' > maxval: 128", prefix);
                             }
 
                             prefix
@@ -213,7 +216,7 @@ impl pf_addr
                 let adr = 
                     match addr.char_indices().nth(pos-1)  // don't count '/'
                     {
-                        None => runtime_error!("can not get char indeces at: '{}' for: '{}'", pos, addr),
+                        None => error_coded!(PfErrCode::InternalError, "can not get char indeces at: '{}' for: '{}'", pos, addr),
                         Some((idx, _)) => &addr[..=idx],
                     };
 
@@ -230,16 +233,16 @@ impl pf_addr
         // prefix only with numeric addresses
         hints.ai_flags |= AI_NUMERICHOST;
 
-        let c = CString::new(adr.to_string()).map_err(|e| map_runtime_error!("{}", e))?;
+        let c = CString::new(adr.to_string()).map_err(|e| map_error_coded!(PfErrCode::InternalError, "{}", e))?;
         let ret_ga = libc::getaddrinfo(c.as_ptr(), std::ptr::null(), &hints as *const _ as *const libc::addrinfo, &mut res);
 
         if ret_ga > 0
         {
-            runtime_error!("getaddrinfo: {}", CStr::from_ptr(libc::gai_strerror(ret_ga)).to_string_lossy());
+            error_coded!(PfErrCode::LibcGetAddrInfo(ret_ga), "getaddrinfo: {}", CStr::from_ptr(libc::gai_strerror(ret_ga)).to_string_lossy());
         }
         else if res == std::ptr::null_mut()
         {
-            runtime_error!("getaddrinfo: returned NULL");
+            error_coded!(PfErrCode::RuntimeAssertion, "getaddrinfo: returned NULL");
         }
 
 
@@ -248,14 +251,14 @@ impl pf_addr
             // drop the result
             libc::freeaddrinfo(res);
 
-            runtime_error!("prefix too long for AF_INET: {}", prefix);
+            error_coded!(PfErrCode::Einval, "prefix too long for AF_INET: {}", prefix);
         }
         else if (*res).ai_family == libc::AF_INET6 && prefix > 128
         {
             // drop the result
             libc::freeaddrinfo(res);
 
-            runtime_error!("prefix too long for AF_INET6: {}", prefix);
+            error_coded!(PfErrCode::Einval, "prefix too long for AF_INET6: {}", prefix);
         }
 
         let q = prefix >> 3;
@@ -292,7 +295,7 @@ impl pf_addr
                 // drop the result
                 libc::freeaddrinfo(res);
 
-                runtime_error!("unknown ai_family: {}", (*res).ai_family);
+                error_coded!(PfErrCode::UnknownAddressFamily((*res).ai_family), "unknown ai_family: {}", (*res).ai_family);
             }
         }
 
@@ -510,8 +513,8 @@ impl node_host
                 hints.ai_socktype = libc::SOCK_DGRAM; //dummy
                 hints.ai_flags = AI_NUMERICHOST;
         
-                let c = CString::new(s.to_string()).map_err(|e| map_runtime_error!("{}", e))?;
-                let service = CString::new("0".to_string()).map_err(|e| map_runtime_error!("{}", e))?;
+                let c = CString::new(s.to_string()).map_err(|e| map_error_coded!(PfErrCode::InternalError, "{}", e))?;
+                let service = CString::new("0".to_string()).map_err(|e| map_error_coded!(PfErrCode::InternalError, "{}", e))?;
 
                 // getaddrinfo(s, "0", &hints, &res
                 let ret_ga = 
@@ -525,7 +528,7 @@ impl node_host
                 else if res == std::ptr::null_mut()
                 {
                     // assertion
-                    runtime_error!("getaddrinfo: returned NULL");
+                    error_coded!(PfErrCode::RuntimeAssertion, "getaddrinfo: returned NULL");
                 }
 
                 //let mut h: node_host= std::mem::zeroed();
@@ -578,7 +581,7 @@ impl node_host
             {
                 match s.char_indices().nth(pos) 
                 {
-                    None => runtime_error!("can not get char indeces at: '{}' for: '{}'", pos, s),
+                    None => error_coded!(PfErrCode::InternalError, "can not get char indeces at: '{}' for: '{}'", pos, s),
                     Some((idx, _)) => 
                     {
                         if &s[idx..] == ":0" // && !strcmp(p, ":0") i.e strcmp(p, ":0") == 0
@@ -605,7 +608,7 @@ impl node_host
             hints.ai_family = libc::PF_UNSPEC;
             hints.ai_socktype = libc::SOCK_STREAM; // DUMMY
 
-            let c_ps = CString::new(s.to_string()).map_err(|e| map_runtime_error!("{}", e))?;
+            let c_ps = CString::new(s.to_string()).map_err(|e| map_error_coded!(PfErrCode::InternalError, "{}", e))?;
 
             // getaddrinfo(s, "0", &hints, &res
             let ret_ga = 
@@ -623,7 +626,7 @@ impl node_host
             }
             else if res == std::ptr::null_mut()
             {
-                runtime_error!("getaddrinfo: returned NULL");
+                error_coded!(PfErrCode::RuntimeAssertion, "getaddrinfo: returned NULL");
             }
 
             let mut got4: libc::c_int = 0;
@@ -730,14 +733,14 @@ impl node_host
                 let str_ip =
                     match s.char_indices().nth(pos) 
                     {
-                        None => runtime_error!("can not get char indeces at: '{}' for: '{}'", pos, s),
+                        None => error_coded!(PfErrCode::InternalError, "can not get char indeces at: '{}' for: '{}'", pos, s),
                         Some((idx, _)) => &s[..idx],
                     };
                 
                 let str_mask = 
                     match s.char_indices().nth(pos+1) // skip '/' = pos + 1 
                     {
-                        None => runtime_error!("can not get char indeces at: '{}' for: '{}'", pos, s),
+                        None => error_coded!(PfErrCode::InternalError, "can not get char indeces at: '{}' for: '{}'", pos, s),
                         Some((idx, _)) => &s[idx..],
                     };
 
@@ -749,12 +752,12 @@ impl node_host
                         {
                             if m > 128
                             {
-                                runtime_error!("invalid netmask: '{}' > 128", m);
+                                error_coded!(PfErrCode::Einval, "invalid netmask: '{}' > 128", m);
                             }
 
                             m
                         },
-                        Err(e) => runtime_error!("invalid netmask: '{}', {}", str_mask, e),
+                        Err(e) => error_coded!(PfErrCode::Einval, "invalid netmask: '{}', {}", str_mask, e),
                     };
 
                 v4mask = mask;
@@ -786,7 +789,7 @@ impl node_host
         }
         else
         {
-            runtime_error!("invalid address: {} not IP or DNS or IF", str_ip);
+            error_coded!(PfErrCode::Einval, "invalid address: {} not IP or DNS or IF", str_ip);
         }
 
         
@@ -835,7 +838,7 @@ impl pfr_addr
                     //C memcpy(&addr.pfra_ip6addr, &n->addr.v.a.addr.v6, sizeof(struct in6_addr));
                     pfr_addr_in_addr{ _pfra_ip6addr: libc::in6_addr{ s6_addr: n.addr.v.a.addr.pfa.addr8 } }
                 },
-                _ => runtime_error!("Unknown AF flag in node_host: '{}'", n.af),
+                _ => error_coded!(PfErrCode::UnknownAddressFamily(n.af as i32), "Unknown AF flag in node_host: '{}'", n.af),
             };
 
         let mut addr =
@@ -853,7 +856,7 @@ impl pfr_addr
             (not > 0 || (addr.pfra_net as i32) != bits) || 
             (addr.pfra_net as i32) > bits
         {
-            runtime_error!("error at expresion: params: is_test: {}, not: {}, addr.pfra_net: {}, bits: {}",
+            error_coded!(PfErrCode::Einval, "error at expresion: params: is_test: {}, not: {}, addr.pfra_net: {}, bits: {}",
                 is_test, not, addr.pfra_net, bits);
         }
 
@@ -915,16 +918,16 @@ impl pfioc_state_kill
 
         let mut res: *mut libc::addrinfo = std::ptr::null_mut();
 
-        let c = CString::new(host.to_string()).map_err(|e| map_runtime_error!("{}", e))?;
+        let c = CString::new(host.to_string()).map_err(|e| map_error_coded!(PfErrCode::InternalError, "{}", e))?;
         let ret_ga = libc::getaddrinfo(c.as_ptr(), std::ptr::null(), std::ptr::null(), &mut res);
 
         if ret_ga > 0
         {
-            runtime_error!("getaddrinfo: {}", CStr::from_ptr(libc::gai_strerror(ret_ga)).to_string_lossy());
+            error_coded!(PfErrCode::LibcGetAddrInfo(ret_ga), "getaddrinfo: {}", CStr::from_ptr(libc::gai_strerror(ret_ga)).to_string_lossy());
         }
         else if res == std::ptr::null_mut()
         {
-            runtime_error!("getaddrinfo: returned NULL");
+            error_coded!(PfErrCode::RuntimeAssertion, "getaddrinfo: returned NULL");
         }
 
         // sockaddr does not contains refereces so it is safe to use zeroed()
@@ -974,7 +977,7 @@ impl pfioc_state_kill
                 else
                 {
                     libc::freeaddrinfo(res);
-                    runtime_error!("unknown address family {}", psk.psk_af);
+                    error_coded!(PfErrCode::UnknownAddressFamily(psk.psk_af as i32), "unknown address family {}", psk.psk_af);
                 }
 
                 //C if (state_killers > 1) {
@@ -991,7 +994,7 @@ impl pfioc_state_kill
                     // init new addrinfo pointer stuct addrinfo * res1;
                     let mut res1: *mut libc::addrinfo = std::ptr::null_mut();
 
-                    let c1 = CString::new(host.to_string()).map_err(|e| map_runtime_error!("{}", e))?;
+                    let c1 = CString::new(host.to_string()).map_err(|e| map_error_coded!(PfErrCode::InternalError, "{}", e))?;
 
                     // call getaddrinfo
                     let ret_ga = libc::getaddrinfo(c1.as_ptr(), std::ptr::null(), std::ptr::null(), &mut res1);
@@ -999,12 +1002,12 @@ impl pfioc_state_kill
                     if ret_ga > 0
                     {
                         libc::freeaddrinfo(res);
-                        runtime_error!("getaddrinfo: {}", CStr::from_ptr(libc::gai_strerror(ret_ga)).to_string_lossy());
+                        error_coded!(PfErrCode::LibcGetAddrInfo(ret_ga), "getaddrinfo: {}", CStr::from_ptr(libc::gai_strerror(ret_ga)).to_string_lossy());
                     }
                     else if res1 == std::ptr::null_mut()
                     {
                         libc::freeaddrinfo(res);
-                        runtime_error!("getaddrinfo: returned NULL");
+                        error_coded!(PfErrCode::RuntimeAssertion, "getaddrinfo: returned NULL");
                     }
 
                     // at this point is must be inited
@@ -1048,7 +1051,7 @@ impl pfioc_state_kill
                             {
                                 libc::freeaddrinfo(res);
                                 libc::freeaddrinfo(res1);
-                                runtime_error!("unknown address family {}", psk.psk_af);
+                                error_coded!(PfErrCode::UnknownAddressFamily(psk.psk_af as i32), "unknown address family {}", psk.psk_af);
                             }
 
                             if is_test == false
@@ -1059,7 +1062,7 @@ impl pfioc_state_kill
                                     libc::freeaddrinfo(res);
                                     libc::freeaddrinfo(res1);
 
-                                    runtime_error!("ioctl DIOCKILLSTATES failed");
+                                    error_coded!(PfErrCode::IoCtlError("DIOCKILLSTATES", IoCtl::DIOCKILLSTATES), "ioctl DIOCKILLSTATES failed");
                                 }
                             }
                             else
@@ -1083,7 +1086,7 @@ impl pfioc_state_kill
                             //pfr_report_error(tbl, &io, "add table");
                             libc::freeaddrinfo(res);
 
-                            runtime_error!("ioctl DIOCRADDTABLES failed");
+                            error_coded!(PfErrCode::IoCtlError("DIOCKILLSTATES", IoCtl::DIOCKILLSTATES), "ioctl DIOCKILLSTATES failed");
                         }
                     }
                     else
@@ -1147,6 +1150,16 @@ impl pfioc_table
 
         return io;
     }
+
+    unsafe fn new_flush(tbl: pfr_table, flags: libc::c_int) -> Self
+    {
+        let mut io: Self = std::mem::zeroed();
+        
+        io.pfrio_flags = flags;
+        io.pfrio_table = tbl;
+
+        return io;
+    }
 }
 
 struct IoCtl{}
@@ -1194,6 +1207,7 @@ impl IoCtl
     const DIOCRDELADDRS: libc::c_ulong = Self::iowr('D', 68, std::mem::size_of::<pfioc_table>());
     const DIOCRTSTADDRS: libc::c_ulong = Self::iowr('D', 73, std::mem::size_of::<pfioc_table>());
     const DIOCKILLSTATES: libc::c_ulong = Self::iowr('D', 41, std::mem::size_of::<pfioc_state_kill>());
+    const DIOCRCLRADDRS: libc::c_ulong = Self::iowr('D', 66, std::mem::size_of::<pfioc_table>());
 }
 
 #[derive(Debug, Clone, Copy)]
@@ -1220,10 +1234,10 @@ impl pfr_table
     {
         if pfrt_name.len() >= PF_TABLE_NAME_SIZE-1
         {
-            runtime_error!("pfrt_name: {} too long: {} >= {}", pfrt_name, pfrt_name.len(), PF_TABLE_NAME_SIZE-1);
+            error_coded!(PfErrCode::Einval, "pfrt_name: {} too long: {} >= {}", pfrt_name, pfrt_name.len(), PF_TABLE_NAME_SIZE-1);
         }
 
-        let c_pfrt_name = CString::new(pfrt_name.to_string()).map_err(|e| map_runtime_error!("{}", e))?;
+        let c_pfrt_name = CString::new(pfrt_name.to_string()).map_err(|e| map_error_coded!(PfErrCode::InternalError, "{}", e))?;
         let len = c_pfrt_name.to_bytes().len();
 
         let mut table = 
@@ -1244,7 +1258,7 @@ impl pfr_table
     {
         if size < 0
         {
-            runtime_error!("EINVAL size: {}", size);
+            error_coded!(PfErrCode::Einval, "pfr_add_tables() EINVAL size: {}", size);
         }
 
         let mut io = pfioc_table::new_add_tables(self.clone(), size, flags);
@@ -1254,7 +1268,7 @@ impl pfr_table
             if libc::ioctl(fd, IoCtl::DIOCRADDTABLES, &mut io as *mut _, std::ptr::null::<libc::c_void>()) != 0
             {
                 //pfr_report_error(tbl, &io, "add table");
-                runtime_error!("ioctl DIOCRADDTABLES failed");
+                error_coded!(PfErrCode::IoCtlError("DIOCRADDTABLES", IoCtl::DIOCRADDTABLES), "ioctl DIOCRADDTABLES failed");
             }
         }
         else
@@ -1278,7 +1292,7 @@ impl pfr_table
     {   
         if b.pfrb_caddr.len() == 0
         {
-            runtime_error!("EINVAL size: {}", b.pfrb_caddr.len());
+            error_coded!(PfErrCode::Einval, "pfr_add_addrs() EINVAL size: {}", b.pfrb_caddr.len());
         }
 
         let mut io = 
@@ -1294,7 +1308,7 @@ impl pfr_table
             if libc::ioctl(fd, IoCtl::DIOCRADDADDRS, &mut io as *mut _, std::ptr::null::<libc::c_void>()) != 0
             {
                 //pfr_report_error(tbl, &io, "add table");
-                runtime_error!("ioctl DIOCRADDADDRS failed");
+                error_coded!(PfErrCode::IoCtlError("DIOCRADDADDRS", IoCtl::DIOCRADDADDRS), "ioctl DIOCRADDADDRS failed");
             }
         }
         else
@@ -1311,7 +1325,7 @@ impl pfr_table
     {   
         if b.pfrb_caddr.len() == 0
         {
-            runtime_error!("EINVAL size: {}", b.pfrb_caddr.len());
+            error_coded!(PfErrCode::Einval, "pfr_del_addrs() EINVAL size: {}", b.pfrb_caddr.len());
         }
 
         let mut io = 
@@ -1325,7 +1339,7 @@ impl pfr_table
         if libc::ioctl(fd, IoCtl::DIOCRDELADDRS, &mut io as *mut _, std::ptr::null::<libc::c_void>()) != 0
         {
             //pfr_report_error(tbl, &io, "add table");
-            runtime_error!("ioctl DIOCRDELADDRS failed");
+            error_coded!(PfErrCode::IoCtlError("DIOCRDELADDRS", IoCtl::DIOCRDELADDRS), "ioctl DIOCRDELADDRS failed");
         }
 
         *ndel = io.pfrio_ndel;
@@ -1337,7 +1351,7 @@ impl pfr_table
     {   
         if b.pfrb_caddr.len() == 0
         {
-            runtime_error!("EINVAL size: {}", b.pfrb_caddr.len());
+            error_coded!(PfErrCode::Einval, "pfr_tst_addrs() EINVAL size: {}", b.pfrb_caddr.len());
         }
 
         let mut io = 
@@ -1351,7 +1365,7 @@ impl pfr_table
         if libc::ioctl(fd, IoCtl::DIOCRTSTADDRS, &mut io as *mut _, std::ptr::null::<libc::c_void>()) != 0
         {
             //pfr_report_error(tbl, &io, "add table");
-            runtime_error!("ioctl DIOCRDELADDRS failed");
+            error_coded!(PfErrCode::IoCtlError("DIOCRTSTADDRS", IoCtl::DIOCRTSTADDRS), "ioctl DIOCRDELADDRS failed");
         }
 
         *nmatch = io.pfrio_nadd; //C *nmatch = io.pfrio_nmatch; #define	pfrio_nmatch	pfrio_nadd
@@ -1359,6 +1373,21 @@ impl pfr_table
         return Ok(());
     }
 
+    unsafe fn pfr_clr_addrs(&mut self, fd: libc::c_int, flags: libc::c_int, ndel: &mut libc::c_int) -> PfResult<()>
+    {
+        let mut io = pfioc_table::new_flush(self.clone(), flags);
+
+        if libc::ioctl(fd, IoCtl::DIOCRCLRADDRS, &mut io as *mut _, std::ptr::null::<libc::c_void>()) != 0
+        {
+            //pfr_report_error(tbl, &io, "add table");
+            error_coded!(PfErrCode::IoCtlError("DIOCRCLRADDRS", IoCtl::DIOCRCLRADDRS), "ioctl DIOCRCLRADDRS failed");
+        }
+
+        *ndel = io.pfrio_ndel; //C *nmatch = io.pfrio_nmatch; #define	pfrio_nmatch	pfrio_nadd
+
+        return Ok(());
+    }
+
     unsafe fn create_table(&mut self, fd: libc::c_int, test: bool) -> PfResult<()>
     {
         let mut nadd: libc::c_int = 0;
@@ -1421,6 +1450,19 @@ impl pfr_table
         return Ok(nmatch as usize == pfrb_size);
     }
 
+    unsafe fn fls_addrs(&mut self, fd: libc::c_int) -> PfResult<i32>
+    {
+        let mut ndel = 0;
+        let flags: libc::c_int = 0;
+
+        self.pfr_clr_addrs(fd, flags, &mut ndel)?;
+
+        #[cfg(feature = "log_to_stdout")]
+        println!("{} addresses removed", ndel);
+
+        return Ok(ndel);
+    }
+
 }
 
 // TODO: switch to Vec and use as_mut_ptr to remove the malloc calls OK
@@ -1466,7 +1508,7 @@ impl PfrBuffer
 
                 match s.char_indices().nth(pos+1)  // skip '!'
                 {
-                    None => runtime_error!("can not get char indeces at: '{}' for: '{}'", pos, s),
+                    None => error_coded!(PfErrCode::InternalError, "can not get char indeces at: '{}' for: '{}'", pos, s),
                     Some((idx, _)) => &s[idx..],
                 }
             }
@@ -1523,19 +1565,19 @@ impl PfrBuffer
             {   
                 if s_path.exists() == true
                 {
-                    runtime_error!("path does not exist: {}", s_path.display());
+                    error_coded!(PfErrCode::IOErr, "path does not exist: {}", s_path.display());
                 }
                 else if s_path.is_dir() == true
                 {
-                    runtime_error!("path is dir: {}", s_path.display());
+                    error_coded!(PfErrCode::IOErr, "path is dir: {}", s_path.display());
                 }
 
-                Box::new(fs::File::open(s_path).map_err(|e| map_runtime_error!("{}", e))?)
+                Box::new(fs::File::open(s_path).map_err(|e| map_error_coded!(PfErrCode::IOErr, "{}", e))?)
             };
 
         //read from stream all
         let mut file_buf = String::new();
-        stream.read_to_string(&mut file_buf).map_err(|e| map_runtime_error!("{}", e))?;
+        stream.read_to_string(&mut file_buf).map_err(|e| map_error_coded!(PfErrCode::IOErr, "{}", e))?;
 
         let token = PfListTokenizer::from_str(&file_buf)?;
         
@@ -1577,11 +1619,12 @@ impl PfrBuffer
     // https://github.com/freebsd/freebsd-src/blob/373ffc62c158e52cde86a5b934ab4a51307f9f2e/sbin/pfctl/pfctl_radix.c#L457
     fn pfr_buf_add(&mut self, addr: pfr_addr) -> PfResult<()>
     {
+        // this should not happen, because it is converted to the enum, but for compat.
         let pfrb_type = self.pfrb_type as i32;
 
         if pfrb_type <= 0 || pfrb_type >= Pfrb::PFRB_MAX as i32
         {
-            runtime_error!("pfrb_type error: {:?}", self.pfrb_type);
+            error_coded!(PfErrCode::Einval, "pfrb_type unknown: {}", self.pfrb_type as i32);
         }
 
         self.pfrb_caddr.push(addr);
@@ -1619,17 +1662,30 @@ pub struct Pf
     test: bool
 }
 
+impl Drop for Pf
+{
+    fn drop(&mut self) 
+    {
+        if self.fd >= 0
+        {
+            unsafe { libc::close(self.fd) };
+        }    
+    }
+}
+
+
 impl Pf
 {
     const DEFAULT_PF_DEVICE: &'static str = "/dev/pf";
 
-    pub fn new(test: bool) -> PfResult<Self>
+    pub 
+    fn new(test: bool) -> PfResult<Self>
     {
         let fd = 
             if test == false
             {
                 open(Self::DEFAULT_PF_DEVICE, OFlag::O_NONBLOCK | OFlag::O_RDWR, Mode::empty())
-                    .map_err(|e| map_runtime_error!("open() failed path {} err: {}", Self::DEFAULT_PF_DEVICE, e))?
+                    .map_err(|e| map_error_coded!(PfErrCode::IOErr, "open() failed path {} err: {}", Self::DEFAULT_PF_DEVICE, e))?
             }
             else
             {
@@ -1641,12 +1697,14 @@ impl Pf
 
     // internal functions to be called from public interface
     // https://github.com/freebsd/freebsd-src/blob/373ffc62c158e52cde86a5b934ab4a51307f9f2e/sbin/pfctl/pfctl.c#L638
-    unsafe fn pfctl_kill_state<H: AsRef<str>>(&self, src: H, dest: Option<H>) -> PfResult<()>
+    pub unsafe 
+    fn pfctl_kill_state<H: AsRef<str>>(&self, src: H, dest: Option<H>) -> PfResult<()>
     {
         return pfioc_state_kill::new(self.fd, self.test, src, dest);
     }
 
-    unsafe fn pfctl_table<T: AsRef<str>>(&self, table_name: T, cmd: PfCmd) -> PfResult<()>
+    pub unsafe 
+    fn pfctl_table<T: AsRef<str>>(&self, table_name: T, cmd: PfCmd) -> PfResult<()>
     {
         let tname = table_name.as_ref();
 
@@ -1656,13 +1714,13 @@ impl Pf
 
         if tname.len() >= PF_TABLE_NAME_SIZE
         {
-            runtime_error!("pf [action]: table name: '{}' is too long!", tname);
+            error_coded!(PfErrCode::Einval, "table name: '{}' is too long!", tname);
         }
 
         // verify that onlu ASCII chars are in tname
         if tname.chars().all(|c| c.is_ascii()) == false
         {
-            runtime_error!("pf [action]: non ascii character in table_name: {}", tname);
+            error_coded!(PfErrCode::Einval, "non ascii character in table_name: {}", tname);
         }
 
         //table.pfrt_name.copy_from_slice(tname.as_bytes());
@@ -1709,6 +1767,10 @@ impl Pf
 
                 table.tst_addrs(self.fd, b)?;
             },
+            PfCmd::Flush => 
+            {
+                table.fls_addrs(self.fd)?;
+            }
         }
 
         return Ok(());
